@@ -1,7 +1,8 @@
 from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import division
-from mwptoolkit.data.dataloader import abstract_dataloader,dataloader_ept,dataloader_multiencdec,pretrain_dataloader,single_equation_dataloader,multi_equation_dataloader,template_dataloader
+from mwptoolkit.data.dataloader import abstract_dataloader,dataloader_ept,dataloader_multiencdec,pretrain_dataloader,single_equation_dataloader,multi_equation_dataloader,template_dataloader,\
+    dataloader_hms
 
 from mwptoolkit.data.dataloader.abstract_dataloader import AbstractDataLoader
 from mwptoolkit.data.dataloader.single_equation_dataloader import SingleEquationDataLoader
@@ -10,3 +11,4 @@ from mwptoolkit.data.dataloader.template_dataloader import TemplateDataLoader
 from mwptoolkit.data.dataloader.dataloader_ept import DataLoaderEPT
 from mwptoolkit.data.dataloader.dataloader_multiencdec import DataLoaderMultiEncDec
 from mwptoolkit.data.dataloader.pretrain_dataloader import PretrainDataLoader
+from mwptoolkit.data.dataloader.dataloader_hms import DataLoaderHMS
