@@ -1,7 +1,8 @@
 from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import division
-from mwptoolkit.data.dataset import abstract_dataset,dataset_ept,dataset_multiencdec,pretrain_dataset,single_equation_dataset,multi_equation_dataset,template_dataset
+from mwptoolkit.data.dataset import abstract_dataset,dataset_ept,dataset_multiencdec,pretrain_dataset,single_equation_dataset,multi_equation_dataset,template_dataset,\
+    dataset_hms
 
 from mwptoolkit.data.dataset.abstract_dataset import AbstractDataset
 from mwptoolkit.data.dataset.single_equation_dataset import SingleEquationDataset
@@ -10,3 +11,4 @@ from mwptoolkit.data.dataset.template_dataset import TemplateDataset
 from mwptoolkit.data.dataset.dataset_ept import DatasetEPT
 from mwptoolkit.data.dataset.dataset_multiencdec import DatasetMultiEncDec
 from mwptoolkit.data.dataset.pretrain_dataset import PretrainDataset
+from mwptoolkit.data.dataset.dataset_hms import DatasetHMS
