@@ -5,6 +5,7 @@
 
 import copy
 import random
+from typing import Tuple, Dict, Any
 
 import torch
 from torch import nn
@@ -12,7 +13,7 @@ from torch.nn import functional as F
 
 from mwptoolkit.module.Encoder.rnn_encoder import BasicRNNEncoder
 from mwptoolkit.module.Decoder.rnn_decoder import BasicRNNDecoder, AttentionalRNNDecoder
-from mwptoolkit.module.Embedder.basic_embedder import BaiscEmbedder
+from mwptoolkit.module.Embedder.basic_embedder import BasicEmbedder
 from mwptoolkit.loss.nll_loss import NLLLoss
 from mwptoolkit.utils.enum_type import NumMask, SpecialTokens
 
@@ -67,11 +68,11 @@ class DNS(nn.Module):
         except:
             self.out_pad_token = None
 
-        self.in_embedder = BaiscEmbedder(self.vocab_size, self.embedding_size, self.dropout_ratio)
+        self.in_embedder = BasicEmbedder(self.vocab_size, self.embedding_size, self.dropout_ratio)
         if self.share_vocab:
             self.out_embedder = self.in_embedder
         else:
-            self.out_embedder = BaiscEmbedder(self.symbol_size, self.embedding_size, self.dropout_ratio)
+            self.out_embedder = BasicEmbedder(self.symbol_size, self.embedding_size, self.dropout_ratio)
 
         self.encoder=BasicRNNEncoder(self.embedding_size,self.hidden_size,self.num_layers,\
                                         self.encoder_rnn_cell_type,self.dropout_ratio,self.bidirectional)
@@ -89,107 +90,99 @@ class DNS(nn.Module):
         pad = self.out_pad_token
         self.loss = NLLLoss(weight, pad)
 
-    def forward(self, seq, seq_length, target=None):
-        batch_size = seq.size(0)
-        device = seq.device
-
-        seq_emb = self.in_embedder(seq)
-        encoder_outputs, encoder_hidden = self.encoder(seq_emb, seq_length)
-
-        if self.bidirectional:
-            encoder_outputs = encoder_outputs[:, :, self.hidden_size:] + encoder_outputs[:, :, :self.hidden_size]
-            if (self.encoder_rnn_cell_type == 'lstm'):
-                encoder_hidden = (encoder_hidden[0][::2].contiguous(), encoder_hidden[1][::2].contiguous())
-            else:
-                encoder_hidden = encoder_hidden[::2].contiguous()
-        if self.encoder.rnn_cell_type == self.decoder.rnn_cell_type:
-            pass
-        elif (self.encoder.rnn_cell_type == 'gru') and (self.decoder.rnn_cell_type == 'lstm'):
-            encoder_hidden = (encoder_hidden, encoder_hidden)
-        elif (self.encoder.rnn_cell_type == 'rnn') and (self.decoder.rnn_cell_type == 'lstm'):
-            encoder_hidden = (encoder_hidden, encoder_hidden)
-        elif (self.encoder.rnn_cell_type == 'lstm') and (self.decoder.rnn_cell_type == 'gru' or self.decoder.rnn_cell_type == 'rnn'):
-            encoder_hidden = encoder_hidden[0]
-        else:
-            pass
-
-        decoder_inputs = self.init_decoder_inputs(target, device, batch_size)
-        if target != None:
-            token_logits = self.generate_t(encoder_outputs, encoder_hidden, decoder_inputs)
-            return token_logits
-        else:
-            all_outputs = self.generate_without_t(encoder_outputs, encoder_hidden, decoder_inputs)
-            return all_outputs
-
-    def calculate_loss(self, batch_data):
-        """Finish forward-propagating, calculating loss and back-propagation.
-        
-        Args:
-            batch_data (dict): one batch data.
-        
-        Returns:
-            float: loss value.
+    def forward(self, seq, seq_length, target=None, output_all_layers=False) -> Tuple[
+            torch.Tensor, torch.Tensor, Dict[str, Any]]:
         """
-        seq = batch_data['question']
-        seq_length = batch_data['ques len']
-        target = batch_data['equation']
 
+        :param torch.Tensor seq: input sequence, shape: [batch_size, seq_length].
+        :param torch.Tensor seq_length: the length of sequence, shape: [batch_size].
+        :param torch.Tensor | None target: target, shape: [batch_size, target_length], default None.
+        :param bool output_all_layers: return output of all layers if output_all_layers is True, default False.
+        :return : token_logits:[batch_size, output_length, output_size], symbol_outputs:[batch_size,output_length], model_all_outputs.
+        :rtype: tuple(torch.Tensor, torch.Tensor, dict)
+        """
         batch_size = seq.size(0)
         device = seq.device
 
         seq_emb = self.in_embedder(seq)
-        encoder_outputs, encoder_hidden = self.encoder(seq_emb, seq_length)
-
-        if self.bidirectional:
-            encoder_outputs = encoder_outputs[:, :, self.hidden_size:] + encoder_outputs[:, :, :self.hidden_size]
-            if (self.encoder_rnn_cell_type == 'lstm'):
-                encoder_hidden = (encoder_hidden[0][::2].contiguous(), encoder_hidden[1][::2].contiguous())
-            else:
-                encoder_hidden = encoder_hidden[::2].contiguous()
-        if self.encoder.rnn_cell_type == self.decoder.rnn_cell_type:
-            pass
-        elif (self.encoder.rnn_cell_type == 'gru') and (self.decoder.rnn_cell_type == 'lstm'):
-            encoder_hidden = (encoder_hidden, encoder_hidden)
-        elif (self.encoder.rnn_cell_type == 'rnn') and (self.decoder.rnn_cell_type == 'lstm'):
-            encoder_hidden = (encoder_hidden, encoder_hidden)
-        elif (self.encoder.rnn_cell_type == 'lstm') and (self.decoder.rnn_cell_type == 'gru' or self.decoder.rnn_cell_type == 'rnn'):
-            encoder_hidden = encoder_hidden[0]
-        else:
-            pass
+        encoder_outputs, encoder_hidden, encoder_layer_outputs = self.encoder_forward(seq_emb, seq_length,
+                                                                                      output_all_layers)
 
         decoder_inputs = self.init_decoder_inputs(target, device, batch_size)
-        token_logits = self.generate_t(encoder_outputs, encoder_hidden, decoder_inputs)
+
+        token_logits, symbol_outputs, decoder_layer_outputs = self.decoder_forward(encoder_outputs, encoder_hidden,
+                                                                                   decoder_inputs, target,
+                                                                                   output_all_layers)
+
+        model_all_outputs = {}
+        if output_all_layers:
+            model_all_outputs['inputs_embedding'] = seq_emb
+            model_all_outputs.update(encoder_layer_outputs)
+            model_all_outputs.update(decoder_layer_outputs)
+
+        return token_logits, symbol_outputs, model_all_outputs
+
+    def calculate_loss(self, batch_data:dict) -> float:
+        """Finish forward-propagating, calculating loss and back-propagation.
+
+        :param batch_data: one batch data.
+        :return: loss value.
+
+        batch_data should include keywords 'question', 'ques len' and 'equation'
+        """
+        seq = torch.tensor(batch_data['question']).to(self.device)
+        seq_length = torch.tensor(batch_data['ques len']).long()
+        target = torch.tensor(batch_data['equation']).to(self.device)
+
+        token_logits, _, _ = self.forward(seq, seq_length, target)
         if self.share_vocab:
             target = self.convert_in_idx_2_out_idx(target)
+
+        outputs = torch.nn.functional.log_softmax(token_logits, dim=-1)
         self.loss.reset()
-        self.loss.eval_batch(token_logits, target.view(-1))
+        self.loss.eval_batch(outputs.view(-1,outputs.size(-1)), target.view(-1))
         self.loss.backward()
 
         return self.loss.get_loss()
 
-    def model_test(self, batch_data):
+    def model_test(self, batch_data:dict) -> tuple:
         """Model test.
-        
-        Args:
-            batch_data (dict): one batch data.
-        
-        Returns:
-            tuple(list,list): predicted equation, target equation.
+
+        :param batch_data: one batch data.
+        :return: predicted equation, target equation.
+
+        batch_data should include keywords 'question', 'ques len', 'equation' and 'num list'.
         """
-        seq = batch_data['question']
-        seq_length = batch_data['ques len']
+        seq = torch.tensor(batch_data['question']).to(self.device)
+        seq_length = torch.tensor(batch_data['ques len']).long()
         num_list = batch_data['num list']
-        target = batch_data['equation']
+        target = torch.tensor(batch_data['equation']).to(self.device)
 
-        batch_size = seq.size(0)
-        device = seq.device
+        _, symbol_outputs, _ = self.forward(seq, seq_length)
+        if self.share_vocab:
+            target = self.convert_in_idx_2_out_idx(target)
+        all_outputs = self.convert_idx2symbol(symbol_outputs, num_list)
+        targets = self.convert_idx2symbol(target, num_list)
+        return all_outputs, targets
 
-        seq_emb = self.in_embedder(seq)
+    def predict(self,batch_data:dict,output_all_layers=False):
+        """
+        predict samples without target.
+        :param dict batch_data: one batch data.
+        :param bool output_all_layers: return all layer outputs of model.
+        :return: token_logits, symbol_outputs, all_layer_outputs
+        """
+        seq = torch.tensor(batch_data['question']).to(self.device)
+        seq_length = torch.tensor(batch_data['ques len']).long()
+        token_logits, symbol_outputs, model_all_outputs = self.forward(seq,seq_length,output_all_layers=output_all_layers)
+        return token_logits, symbol_outputs, model_all_outputs
+
+    def encoder_forward(self, seq_emb, seq_length, output_all_layers=False):
         encoder_outputs, encoder_hidden = self.encoder(seq_emb, seq_length)
 
         if self.bidirectional:
             encoder_outputs = encoder_outputs[:, :, self.hidden_size:] + encoder_outputs[:, :, :self.hidden_size]
-            if (self.encoder_rnn_cell_type == 'lstm'):
+            if self.encoder_rnn_cell_type == 'lstm':
                 encoder_hidden = (encoder_hidden[0][::2].contiguous(), encoder_hidden[1][::2].contiguous())
             else:
                 encoder_hidden = encoder_hidden[::2].contiguous()
@@ -199,70 +192,55 @@ class DNS(nn.Module):
             encoder_hidden = (encoder_hidden, encoder_hidden)
         elif (self.encoder.rnn_cell_type == 'rnn') and (self.decoder.rnn_cell_type == 'lstm'):
             encoder_hidden = (encoder_hidden, encoder_hidden)
-        elif (self.encoder.rnn_cell_type == 'lstm') and (self.decoder.rnn_cell_type == 'gru' or self.decoder.rnn_cell_type == 'rnn'):
+        elif (self.encoder.rnn_cell_type == 'lstm') and (
+                self.decoder.rnn_cell_type == 'gru' or self.decoder.rnn_cell_type == 'rnn'):
             encoder_hidden = encoder_hidden[0]
         else:
             pass
-        decoder_inputs = self.init_decoder_inputs(target=None, device=device, batch_size=batch_size)
-        all_outputs = self.generate_without_t(encoder_outputs, encoder_hidden, decoder_inputs)
-        if self.share_vocab:
-            target = self.convert_in_idx_2_out_idx(target)
-        all_outputs = self.convert_idx2symbol(all_outputs, num_list)
-        targets = self.convert_idx2symbol(target, num_list)
-        return all_outputs, targets
 
-    def generate_t(self, encoder_outputs, encoder_hidden, decoder_inputs):
+        all_layer_outputs = {}
+        if output_all_layers:
+            all_layer_outputs['encoder_outputs'] = encoder_outputs
+            all_layer_outputs['encoder_hidden'] = encoder_hidden
+        return encoder_outputs, encoder_hidden, all_layer_outputs
+
+    def decoder_forward(self, encoder_outputs, encoder_hidden, decoder_inputs, target=None, output_all_layers=False):
         with_t = random.random()
-        seq_len = decoder_inputs.size(1)
+        seq_len = decoder_inputs.size(1) if target is not None else self.max_gen_len
         decoder_hidden = encoder_hidden
         decoder_input = decoder_inputs[:, 0, :].unsqueeze(1)
+        decoder_outputs = []
         token_logits = []
+        outputs = []
         output = []
         for idx in range(seq_len):
-            if with_t < self.teacher_force_ratio:
+            if target is not None and with_t < self.teacher_force_ratio:
                 decoder_input = decoder_inputs[:, idx, :].unsqueeze(1)
             if self.attention:
                 decoder_output, decoder_hidden = self.decoder(decoder_input, decoder_hidden, encoder_outputs)
             else:
                 decoder_output, decoder_hidden = self.decoder(decoder_input, decoder_hidden)
-            #attn_list.append(attn)
             step_output = decoder_output.squeeze(1)
             token_logit = self.generate_linear(step_output)
-            predict = torch.nn.functional.log_softmax(token_logit, dim=1)
             output = self.rule_filter_(output, token_logit)
-            token_logits.append(predict)
+            decoder_outputs.append(decoder_output)
+            token_logits.append(token_logit)
+            outputs.append(output)
 
             if self.share_vocab:
                 output_ = self.convert_out_idx_2_in_idx(output)
                 decoder_input = self.out_embedder(output_)
             else:
                 decoder_input = self.out_embedder(output)
+        decoder_outputs = torch.stack(decoder_outputs, dim=1)
         token_logits = torch.stack(token_logits, dim=1)
-        token_logits = token_logits.view(-1, token_logits.size(-1))
-        return token_logits
-
-    def generate_without_t(self, encoder_outputs, encoder_hidden, decoder_input):
-        all_outputs = []
-        decoder_hidden = encoder_hidden
-        output = []
-        for idx in range(self.max_gen_len):
-            if self.attention:
-                decoder_output, decoder_hidden = self.decoder(decoder_input, decoder_hidden, encoder_outputs)
-            else:
-                decoder_output, decoder_hidden = self.decoder(decoder_input, decoder_hidden)
-            #attn_list.append(attn)
-            step_output = decoder_output.squeeze(1)
-            token_logits = self.generate_linear(step_output)
-            output = self.rule_filter_(output, token_logits)
-
-            all_outputs.append(output)
-            if self.share_vocab:
-                output_ = self.convert_out_idx_2_in_idx(output)
-                decoder_input = self.out_embedder(output_)
-            else:
-                decoder_input = self.out_embedder(output)
-        all_outputs = torch.cat(all_outputs, dim=1)
-        return all_outputs
+        outputs = torch.stack(outputs,dim=1)
+        all_layer_outputs = {}
+        if output_all_layers:
+            all_layer_outputs['decoder_outputs'] = decoder_outputs
+            all_layer_outputs['token_logits'] = token_logits
+            all_layer_outputs['outputs'] = outputs
+        return token_logits, outputs, all_layer_outputs
 
     def init_decoder_inputs(self, target, device, batch_size):
         pad_var = torch.LongTensor([self.sos_token_idx] * batch_size).to(device).view(batch_size, 1)
