@@ -1,10 +1,11 @@
 from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import division
-from mwptoolkit.model.Seq2Tree import gts,sausolver,treelstm,trnn,tsn
+from mwptoolkit.model.Seq2Tree import gts,sausolver,treelstm,trnn,tsn,berttd
 
 from mwptoolkit.model.Seq2Tree.gts import GTS
 from mwptoolkit.model.Seq2Tree.sausolver import SAUSolver
 from mwptoolkit.model.Seq2Tree.treelstm import TreeLSTM
 from mwptoolkit.model.Seq2Tree.trnn import TRNN
 from mwptoolkit.model.Seq2Tree.tsn import TSN
+from mwptoolkit.model.Seq2Tree.berttd import BertTD
